/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *                                                                                  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *                                                                                  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
/*
 * form_post.c - Functions to help with handling POSTs
 *
 * All blame to Mike McCool
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libadminutil/admutil.h"
#include "dbtadmutil.h"

extern Resource *admutil_i18nResource;
extern char *admutil_acceptLang;

#ifndef BIG_LINE
#define BIG_LINE 1024
#endif

static char **input;

#ifdef XP_WIN32
PR_IMPLEMENT(char *)
Get_QueryNT(void)
{
    char *qs = getenv("QUERY_STRING");
    if(qs && (*qs == '\0'))
        qs = NULL;
    return qs;
}
#endif  /* XP_WIN32 */
 
PR_IMPLEMENT(char **)
get_input_ptr(void)
{
    return input;
}

PR_IMPLEMENT(void)
form_unescape(char *str) 
{
    register int x = 0, y = 0;
    int l = PL_strlen(str);
    char digit;

    while(x < l)  {
        if((str[x] == '%') && (x < (l - 2)))  {
            ++x;
            digit = (str[x] >= 'A' ? 
                         ((str[x] & 0xdf) - 'A')+10 : (str[x] - '0'));
            digit *= 16;

            ++x;
            digit += (str[x] >= 'A' ? 
                         ((str[x] & 0xdf) - 'A')+10 : (str[x] - '0'));

            str[y] = digit;
        } 
        else if(str[x] == '+')  {
            str[y] = ' ';
        } else {
            str[y] = str[x];
        }
        x++;
        y++;
    }
    str[y] = '\0';
}

/*
 * form_unescape_url_escape_html -- 1) unescape escaped chars in URL;
 *                                  2) escape unsecure chars for scripts
 * 1) "%##" is converted to one character which value is ##; so is '+' to ' '
 * 2) <, >, &, ", ' are escaped with "&XXX;" format
 */
PR_IMPLEMENT(char *)
form_unescape_url_escape_html(char *str) 
{
    register int x = 0, y = 0;
    int l = 0;
    char digit;
    char *rstr = NULL;

    if (NULL == str) {
        return NULL;
    }

    /* to allocate enough space for the escaped characters */
    for (x = 0, y = 0; str[x] != '\0'; x++) {
        if (('<' == str[x]) || ('>' == str[x]))
            y += 3;
        else if (('&' == str[x]) || ('\'' == str[x]))
            y += 4;
        else if ('"' == str[x])
            y += 5;
    }

    if (0 < y) {
        rstr = (char *)PR_Malloc(x + y + 1);
    } else {
        rstr = PL_strdup(str);
    }
    l = x; /* length of str */

    if (NULL == rstr) {
        if (admutil_i18nResource) {
            char buf[BUFSIZ];
            rpt_err(MEMORY_ERROR,
                    NULL,
                    (char*)res_getstring(admutil_i18nResource,
                                         DBT_formPost_PostStdinErr,
                                         admutil_acceptLang, buf, sizeof(buf),
                                         NULL), NULL);
        } else {
            rpt_err(MEMORY_ERROR,
                NULL, 
                "Could not allocate enough memory to read in the POST parameters.",
                NULL);
        }
        return NULL;
    }

    for (x = 0, y = 0; x < l; x++, y++) {
        if (('%' == str[x]) && (x < (l - 2))) {
            ++x;
            digit = (str[x] >= 'A' ? 
                         ((str[x] & 0xdf) - 'A') + 10 : (str[x] - '0'));
            digit *= 16;

            ++x;
            digit += (str[x] >= 'A' ? 
                         ((str[x] & 0xdf) - 'A') + 10 : (str[x] - '0'));

            rstr[y] = digit;
        } else if (str[x] == '+')  {
            rstr[y] = ' ';
        } else if ('<' == str[x]) {
            memcpy(&rstr[y], "&lt;", 4);
            y += 3;
        } else if ('>' == str[x]) {
            memcpy(&rstr[y], "&gt;", 4);
            y += 3;
        } else if ('&' == str[x]) {
            memcpy(&rstr[y], "&amp;", 5);
            y += 4;
        } else if ('"' == str[x]) {
            memcpy(&rstr[y], "&quot;", 6);
            y += 5;
        } else if ('\'' == str[x]) {
            memcpy(&rstr[y], "&#39;", 5);
            y += 4;
        } else {
            rstr[y] = str[x];
        }
    }
    rstr[y] = '\0';
    return rstr;
}

PR_IMPLEMENT(int)
post_begin(FILE *in) 
{
    char *vars = NULL, *tmp = NULL;
    int cl;
    char buf1[BUFSIZ];
    char buf2[BUFSIZ];
    int rc = 0;

    if(!(tmp = getenv("CONTENT_LENGTH"))) {
        if (admutil_i18nResource) {
            rpt_err(INCORRECT_USAGE,
                    (char*)res_getstring(admutil_i18nResource,
                                         DBT_formPost_Browser_err,
                                         admutil_acceptLang, buf1, sizeof(buf1), NULL),
                    (char*)res_getstring(admutil_i18nResource,
                                         DBT_formPost_Browser_errDetail,
                                         admutil_acceptLang, buf2, sizeof(buf2), NULL),
                    NULL);
        }
        else {
            rpt_err(INCORRECT_USAGE, 
                    "Browser Error", 
                    "Your browser sent no content length with a POST command. Please be sure to use a fully compliant browser.",
                    NULL);
        }
        rc = 1;
    }
        
    cl = atoi(tmp);

    if (!(vars = (char *)PR_Malloc(cl+1))) {
      if (admutil_i18nResource) {
          rpt_err(MEMORY_ERROR,
                  NULL,
                  (char*)res_getstring(admutil_i18nResource,
                                       DBT_formPost_PostStdinErr,
                                       admutil_acceptLang, buf1, sizeof(buf1), NULL),
                  NULL);
      }
      else {
        rpt_err(MEMORY_ERROR,
                NULL, 
                "Could not allocate enough memory to read in the POST parameters.",
                NULL);
      }
      rc = 1;
    }        

    if( !(fread(vars, 1, cl, in)) ) {
      if (admutil_i18nResource) {
          rpt_err(SYSTEM_ERROR,
                  NULL,
                  (char*)res_getstring(admutil_i18nResource,
                                       DBT_formPost_PostStdinErr,
                                       admutil_acceptLang, buf1, sizeof(buf1), NULL),
                  NULL);
      }
      else {
        rpt_err(SYSTEM_ERROR,
                NULL, 
                "The POST variables could not be read from stdin.",
                NULL);
      }
      rc = 1;
    }

    vars[cl] = '\0';

    input = string_to_vec(vars);
    PL_strfree(vars); /* string_to_vec dups it */
    return rc;
}

PR_IMPLEMENT(void)
get_begin(char *qs)
{
    input = string_to_vec(qs);
}

PR_IMPLEMENT(char **)
string_to_vec(char *in)
{
    char **ans = NULL;
    int vars = 0;
    register int x = 0;
    char *tmp;
    char buf[BUFSIZ];

    if (!(in = PL_strdup(in))) {
      if (admutil_i18nResource) {
          rpt_err(MEMORY_ERROR,
                  NULL,
                  (char*)res_getstring(admutil_i18nResource,
                                       DBT_formPost_PostStdinErr,
                                       admutil_acceptLang, buf, sizeof(buf), NULL),
                  NULL);
      }
      else {
        rpt_err(MEMORY_ERROR,
                NULL, 
                "Could not allocate enough memory to read in the POST parameters.",
                NULL);
      }
      return ans;
    }        

    while(in[x])
        if(in[x++]=='=')
            vars++;
    
    ans = new_strlist(vars+1);
  
    x=0;
    tmp = strtok(in, "&");
    if (!tmp || !strchr(tmp, '=')) { /* error, bail out */
        PR_Free(in);
        return(ans);
    }

    if (!(ans[x++] = form_unescape_url_escape_html(tmp))) {
        /* could not allocate enough memory */
        PR_Free(in);
        return ans;
    }

    while((tmp = strtok(NULL, "&")))  {
        if (!strchr(tmp, '=')) {
            PR_Free(in);
            return ans;
        }
        if (!(ans[x++] = form_unescape_url_escape_html(tmp))) {
            /* could not allocate enough memory */
            PR_Free(in);
            return ans;
        }
    }

    PR_Free(in);

    return(ans);
}

PR_IMPLEMENT(char *)
get_cgi_var(char *varname, char *elem_id, char *bongmsg)
{
    register int x = 0;
    int len = PL_strlen(varname);
    char *ans = NULL;
    char buf[BUFSIZ];
   
    while(input[x])  {
    /*  We want to get rid of the =, so len, len+1 */
        if((!strncmp(input[x], varname, len)) && (*(input[x]+len) == '='))  {
            if (!(ans = PL_strdup(input[x] + len + 1))) {
                if (admutil_i18nResource) {
                    rpt_err(MEMORY_ERROR,
                            NULL,
                            (char*)res_getstring(admutil_i18nResource,
                                                 DBT_formPost_PostStdinErr,
                                                 admutil_acceptLang, buf, sizeof(buf), NULL),
                            NULL);
                }
                else {
                    rpt_err(MEMORY_ERROR,
                            NULL, 
                            "Could not allocate enough memory to get the parameter.",
                            NULL);
                }
                return ans;
            }
        
            if(!strcmp(ans, ""))
                ans = NULL;
            break;
        }  else
            x++;
    }
    if(ans == NULL)  {
        if ((bongmsg) && PL_strlen(bongmsg))
            rpt_err(ELEM_MISSING, elem_id, bongmsg, NULL);
        else
            return NULL;
    }
    else
        return(ans);
    /* shut up gcc */
    return NULL;
}

PR_IMPLEMENT(long)
get_cgi_long(char *varname)
{
    char *ans = get_cgi_var(varname, NULL, NULL);
 
    return ans ? atol(ans) : -1L;
}

PR_IMPLEMENT(int)
get_cgi_int(char *varname)
{
    return (int)get_cgi_long(varname);
}

PR_IMPLEMENT(int)
get_cgi_bool(char *varname)
{
    char *ans = get_cgi_var(varname, NULL, NULL);
 
    return (ans && *ans && (*ans - '0')) ? 1 : 0;
}

PR_IMPLEMENT(char **)
get_cgi_multiple(char *varname, char *elem_id, char *bongmsg)
{
    register int n, x;
    int len = PL_strlen(varname);
    char **ans = NULL;
    char buf[BUFSIZ];

    for(n=0; input[n]; n++);
    ans = new_strlist(n + 1);

    for(x=n=0; input[x]; x++)  {
        if ((!strncmp(input[x], varname, len)) &&
        (*(input[x]+len) == '=') &&
        (*(input[x]+len+1))) {
            if (!(ans[n] = PL_strdup(input[x] + len + 1))) {
                if (admutil_i18nResource) {
                    rpt_err(MEMORY_ERROR,
                            NULL,
                            (char*)res_getstring(admutil_i18nResource,
                                                 DBT_formPost_PostStdinErr,
                                                 admutil_acceptLang, buf, sizeof(buf), NULL),
                            NULL);
                }
                else {
                    rpt_err(MEMORY_ERROR,
                            NULL, 
                            "Could not allocate enough memory to get the parameter.",
                            NULL);
                }
                return ans;
            }
        n++;
        }
    }
    ans[n] = NULL;

    if (!n)  {
        PR_Free(ans);
        if(bongmsg)
            rpt_err(ELEM_MISSING, elem_id, bongmsg, NULL);
        else
            return NULL;
    }
    else {
        return ans;
    }
    /* shut up gcc */
    return NULL;
}


PR_IMPLEMENT(void)
rm_trail_slash(char *target)
{
    int l;
    
    if(!target) return;

    l = PL_strlen(target) - 1;

    if((target[l] == '/') && (l != 0))
        target[l] = '\0';
}

/* Compresses spaces only. */
PR_IMPLEMENT(void)
compress_spaces(char *source) 
{
    register int x, y;

    if(!source) return;

    for(x=0, y=0; source[x]; x++)  {
        if(source[x] != ' ')
             source[y++] = source[x];
    }
    source[y] = '\0';
}

/* Compresses spaces and replaces commas with an or list */
PR_IMPLEMENT(char *)
compress_and_replace(char *source) 
{
    char *answer;
    int hascomma, x, y;

    if(!source) return source;

    hascomma = (strchr(source, ',')) ? 1 : 0;

    /* + 2 = 1 for trailing \0 and one put in by Aruna for NT */
    answer = (char *)PR_Malloc(PL_strlen(source) + 2);
    if (NULL == answer)
        return NULL;

    for(x=0, y=0; source[x]; x++)  {
        if(source[x] == ',')
             answer[y++] = '|';
        else if(source[x] != ' ')
             answer[y++] = source[x];
        answer[y] = '\0';
    }
    if(!hascomma)
        return(answer);
    else {
        char *newanswer = PR_smprintf("(%s)", answer);
        free_str(answer);
        return(newanswer);
    }
}

